#
# Copyright 2018 Quantopian, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import time

from pandas.tseries.holiday import Holiday, GoodFriday, EasterMonday
from pytz import timezone

from .common_holidays import (
    new_years_day,
    ascension_day,
    whit_monday,
    european_labour_day,
    christmas_eve,
    christmas,
    boxing_day,
    new_years_eve,
)
from .trading_calendar import HolidayCalendar, TradingCalendar, WEEKDAYS

NewYearsDay = new_years_day()

AscensionDay = ascension_day(end_date='2001')
WhitMonday = whit_monday(end_date='2002')

LabourDay = european_labour_day()

BelgiumIndependenceDay = Holiday(
    'Belgium Independence Day',
    month=7,
    day=21,
    end_date='2002',
)

ChristmasEve = christmas_eve(days_of_week=WEEKDAYS)
Christmas = christmas()
BoxingDay = boxing_day()

NewYearsEveBefore2002 = new_years_eve(end_date='2002')
NewYearsEveInOrAfter2002 = new_years_eve(
    start_date='2002',
    days_of_week=WEEKDAYS,
)


class XBRUExchangeCalendar(TradingCalendar):
    """
    Calendar for the Euronext Brussels exchange, and the primary calendar for
    the country of Belgium.

    Open Time: 9:00 AM, CET (Central European Time)
    Close Time: 5:30 PM, CET (Central European Time)

    Regularly-Observed Holidays:
      - New Year's Day
      - Good Friday
      - Easter Monday
      - Labour Day
      - Christmas Day
      - Boxing Day

    Early Closes:
      - Christmas Eve
      - New Year's Eve

    Other countries on the Euronext:
      - France
      - Netherlands
      - Portugal
    """
    # Source: https://www.euronext.com/en/calendars-hours
    regular_early_close = time(14, 5)

    @property
    def name(self):
        # Euronext Brussels
        return 'XBRU'

    tz = timezone('Europe/Brussels')

    open_times = (
        (None, time(9, 1)),
    )

    close_times = (
        (None, time(17, 30)),
    )

    @property
    def regular_holidays(self):
        return HolidayCalendar([
            NewYearsDay,
            GoodFriday,
            EasterMonday,
            AscensionDay,
            WhitMonday,
            LabourDay,
            BelgiumIndependenceDay,
            Christmas,
            BoxingDay,
            NewYearsEveBefore2002,
        ])

    @property
    def special_closes(self):
        return [
            (
                self.regular_early_close,
                HolidayCalendar([ChristmasEve, NewYearsEveInOrAfter2002]),
            ),
        ]



from unittest import TestCase
import pandas as pd

from .test_trading_calendar import ExchangeCalendarTestBase
from .test_utils import T
from trading_calendars.exchange_calendar_xshg import XSHGExchangeCalendar


class XSHGCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xshg'
    calendar_class = XSHGExchangeCalendar

    # Shanghai stock exchange is open from 9:30 am to 3pm
    # (for now, ignoring lunch break)
    MAX_SESSION_HOURS = 5.5

    HAVE_EARLY_CLOSES = False

    MINUTE_INDEX_TO_SESSION_LABELS_END = pd.Timestamp('2011-04-07', tz='UTC')

    def test_normal_year(self):
        expected_holidays_2017 = [
            T("2017-01-02"),
            T("2017-01-27"),
            T("2017-01-30"),
            T("2017-01-31"),
            T("2017-02-01"),
            T("2017-02-02"),
            T("2017-04-03"),
            T("2017-04-04"),
            T("2017-05-01"),
            T("2017-05-29"),
            T("2017-05-30"),
            T("2017-10-02"),
            T("2017-10-03"),
            T("2017-10-04"),
            T("2017-10-05"),
            T("2017-10-06"),
        ]

        for session_label in expected_holidays_2017:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_constrain_construction_dates(self):
        # the XSHG calendar currently goes from 1999 to 2025, inclusive.
        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('1998-12-31'), T('2005-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XSHG holidays are only recorded back to 1999,'
                ' cannot instantiate the XSHG calendar back to 1998.'
            )
        )

        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('2005-01-01'), T('2026-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XSHG holidays are only recorded to 2025,'
                ' cannot instantiate the XSHG calendar for 2026.'
            )
        )



from unittest import TestCase

from .test_trading_calendar import ExchangeCalendarTestBase
from .test_utils import T
from trading_calendars.exchange_calendar_xses import XSESExchangeCalendar


class XSESCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xses'
    calendar_class = XSESExchangeCalendar

    # Singapore stock exchange is open from 9am to 5pm
    # (for now, ignoring lunch break)
    MAX_SESSION_HOURS = 8

    HAVE_EARLY_CLOSES = False

    def test_normal_year(self):
        expected_holidays_2017 = [
            T("2017-01-02"),
            T("2017-01-30"),
            T("2017-04-14"),
            T("2017-05-01"),
            T("2017-05-10"),
            T("2017-06-26"),
            T("2017-08-09"),
            T("2017-09-01"),
            T("2017-10-18"),
            T("2017-12-25"),
        ]

        for session_label in expected_holidays_2017:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_constrain_construction_dates(self):
        # the XSES calendar currently goes from 1999 to 2025, inclusive.
        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('1985-12-31'), T('2005-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XSES holidays are only recorded back to 1986,'
                ' cannot instantiate the XSES calendar back to 1985.'
            )
        )

        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('2005-01-01'), T('2021-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XSES holidays are only recorded to 2020,'
                ' cannot instantiate the XSES calendar for 2021.'
            )
        )



from datetime import time
import pandas as pd
from pytz import timezone
from .trading_calendar import TradingCalendar, HolidayCalendar
from .common_holidays import new_years_day
from pandas.tseries.holiday import (
  Holiday,
  next_monday,
)

MoscowNewYears = Holiday(
  'Moscow New Years Day',
  month=1,
  day=1,
  observance=next_monday
)

OrthodoxChristmas = Holiday(
  'Orthodox Christmas',
  month=1,
  day=7,
  observance=next_monday
)

DefenderOfTheFatherland = Holiday(
  'Defender of the Fatherland Day',
  month=2,
  day=23,
  observance=next_monday
)

InternationalWomensDay = Holiday(
  "International Women's Day",
  month=3,
  day=28,
  observance=next_monday
)

SpringAndLaborDay = Holiday(
  "Spring and Labor Day",
  month=5,
  day=1,
  observance=next_monday
)

VictoryDay = Holiday(
  "Victory Day",
  month=5,
  day=9,
  observance=next_monday
)

RussiaDay = Holiday(
  "Russia Day",
  month=6,
  day=12,
  observance=next_monday
)

UnityDay = Holiday(
  "Unity Day",
  month=11,
  day=4,
  observance=next_monday
)


class MISXExchangeCalendar(TradingCalendar):
    """
    Exchange calendar for the Moscow Stock Exchange (MISX, MOEX).

    Open time: 10:00 Europe/Moscow
    Close time: 18:45 Europe/Moscow
    """
    name = "MISX"
    tz = timezone("Europe/Moscow")
    open_times = (
      (None, time(10)),
    )
    close_times = (
      (None, time(18, 45)),
    )

    @property
    def regular_holidays(self):
      return HolidayCalendar([
        MoscowNewYears,
        OrthodoxChristmas,
        DefenderOfTheFatherland,
        InternationalWomensDay,
        SpringAndLaborDay,
        VictoryDay,
        RussiaDay,
        UnityDay
      ])



from unittest import TestCase

from .test_trading_calendar import ExchangeCalendarTestBase
from .test_utils import T
from trading_calendars.exchange_calendar_xbom import XBOMExchangeCalendar


class XBOMCalendarTestCase(ExchangeCalendarTestBase, TestCase):

    answer_key_filename = 'xbom'
    calendar_class = XBOMExchangeCalendar

    # BSE is open from 9:15 am to 3:30 pm
    MAX_SESSION_HOURS = 6.25

    HAVE_EARLY_CLOSES = False

    def test_normal_year(self):
        expected_holidays_2017 = [
            T('2017-01-26'),
            T('2017-02-24'),
            T('2017-03-13'),
            T('2017-04-04'),
            T('2017-04-14'),
            T('2017-05-01'),
            T('2017-06-26'),
            T('2017-08-15'),
            T('2017-08-25'),
            T('2017-10-02'),
            T('2017-10-20'),
            T('2017-12-25'),
        ]

        for session_label in expected_holidays_2017:
            self.assertNotIn(session_label, self.calendar.all_sessions)

    def test_constrain_construction_dates(self):
        # the XBOM calendar currently goes from 1997 to 2020, inclusive.
        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('1996-12-31'), T('1998-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XBOM holidays are only recorded back to 1997,'
                ' cannot instantiate the XBOM calendar back to 1996.'
            )
        )

        with self.assertRaises(ValueError) as e:
            self.calendar_class(T('1998-01-01'), T('2021-01-01'))

        self.assertEqual(
            str(e.exception),
            (
                'The XBOM holidays are only recorded to 2020,'
                ' cannot instantiate the XBOM calendar for 2021.'
            )
        )
